﻿using System;
using System.Threading;

using wclCommon;
using wclWiFi;

namespace WiFiDirect
{
    class Program
    {
        private class WiFiDirect
        {
            private ManualResetEvent TerminateEvent;
            private Thread Thread;

            private void wclWiFiDirectAdvertiserStarted(Object Sender, EventArgs e)
            {
                Console.WriteLine("WiFi Direct Advertiser has been started");
            }

            private void wclWiFiDirectAdvertiserStopped(Object Sender, EventArgs e)
            {
                Console.WriteLine("WiFi Direct Advertiser has been stopped");
            }

            private void wclWiFiDirectAdvertiserAcceptDevice(Object Sender, wclWiFiDirectDevice Device,
                out Boolean Accept)
            {
                Console.WriteLine("Device " + Device.Name + " requires connection. Accept.");
                Accept = true;
            }

            private void wclWiFiDirectAdvertiserDeviceConnected(Object Sender, wclWiFiDirectDevice Device,
                Int32 Error)
            {
                if (Error != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Device " + Device.Name + " connect error: 0x" + Error.ToString("X8"));
                else
                {
                    Console.WriteLine("Device " + Device.Name + " has been connected");
                    Console.WriteLine("  Local IP: " + Device.LocalAddress);
                    Console.WriteLine("  Remote IP: " + Device.RemoteAddress);
                }
            }

            private void wclWiFiDirectAdvertiserDeviceDisconnected(Object Sender, wclWiFiDirectDevice Device,
                Int32 Reason)
            {
                Console.WriteLine("Device " + Device.Name + " has been disconnected by reason: 0x" +
                    Reason.ToString("X8"));
            }

            private void wclWiFiDirectAdvertiserPairCompleted(Object Sender, wclWiFiDirectDevice Device, Int32 Result)
            {
                Console.WriteLine("Pairing with device " + Device.Name + " has been completed with result: 0x" +
                    Result.ToString("X8"));
            }

            private void wclWiFiDirectAdvertiserPairConfirm(Object Sender, wclWiFiDirectDevice Device,
                out Boolean Confirm)
            {
                Console.WriteLine("Pairing with device " + Device.Name + " has been confirmed");
                Confirm = true;
            }

            private void wclWiFiDirectAdvertiserPairDisplayPin(Object Sender, wclWiFiDirectDevice Device, String Pin)
            {
                Console.WriteLine("To pair with device " + Device.Name + " type this PIN: " + Pin +
                    " on device keyboard");
            }

            private void wclWiFiDirectAdvertiserPairGetParams(Object Sender, wclWiFiDirectDevice Device, out Int16 GroupOwnerIntent,
                out wclWiFiDirectConfigurationMethod ConfigurationMethods, out wclWiFiDirectPairingProcedure PairingProcedure)
            {
                Console.WriteLine("Connection is accepted with following parameters:");
                Console.WriteLine("  Pairing procedure: Group Owner Negotiation");
                Console.WriteLine("  GroupOwnerIntent: default (14)");
                Console.WriteLine("  Configuration (pairing) method: Push Button (Configm Only)");

                PairingProcedure = wclWiFiDirectPairingProcedure.ppGroupOwnerNegotiation;
                GroupOwnerIntent = 14;
                ConfigurationMethods = wclWiFiDirectConfigurationMethod.cmPushButton;
            }

            private void wclWiFiDirectAdvertiserPairProvidePin(Object Sender, wclWiFiDirectDevice Device, out String Pin)
            {
                Console.WriteLine("Device requires pairing with entering PIN on Advertiser side. This type of pairing is not supported in this demo");
                Pin = "";
            }

            private void ThreadProc()
            {
                wclWiFiDirectAdvertiser Advertiser = new wclWiFiDirectAdvertiser();
                Advertiser.OnStarted += wclWiFiDirectAdvertiserStarted;
                Advertiser.OnStopped += wclWiFiDirectAdvertiserStopped;
                Advertiser.OnAcceptDevice += wclWiFiDirectAdvertiserAcceptDevice;
                Advertiser.OnDeviceConnected += wclWiFiDirectAdvertiserDeviceConnected;
                Advertiser.OnDeviceDisconnected += wclWiFiDirectAdvertiserDeviceDisconnected;
                Advertiser.OnPairCompleted += wclWiFiDirectAdvertiserPairCompleted;
                Advertiser.OnPairConfirm += wclWiFiDirectAdvertiserPairConfirm;
                Advertiser.OnPairDisplayPin += wclWiFiDirectAdvertiserPairDisplayPin;
                Advertiser.OnPairProvidePin += wclWiFiDirectAdvertiserPairProvidePin;
                Advertiser.OnPairGetParams += wclWiFiDirectAdvertiserPairGetParams;

                Advertiser.AutonomousGroupOwnerEnabled = false;
                Advertiser.Discoverability = wclWiFiDirectAdvertisementDiscoverability.adNormal;
                Advertiser.Legacy = false;

                Int32 Res = Advertiser.Start();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Failed to start WiFi Direct Advertiser: 0x" + Res.ToString("X8"));

                wclMessageBroadcaster.Wait(TerminateEvent);

                if (Advertiser.Active)
                {
                    Res = Advertiser.Stop();
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        Console.WriteLine("Failed to stop WiFi Direct Advertiser: 0x" + Res.ToString("X8"));
                }

                Advertiser = null;
            }

            public WiFiDirect()
            {
                TerminateEvent = null;
                Thread = null;
            }

            public void Start()
            {
                TerminateEvent = new ManualResetEvent(false);
                Thread = new Thread(ThreadProc);
                Thread.Start();
            }

            public void Stop()
            {
                TerminateEvent.Set();
                Thread.Join();
                Thread = null;
                TerminateEvent.Close();
            }
        };

        static void Main(string[] args)
        {
            wclMessageBroadcaster.SetSyncMethod(wclMessageSynchronizationKind.skApc);

            Console.Clear();
            Console.WriteLine("WIFI DIRECT ADVERTISER DEMO");
            Console.WriteLine("===========================");
            Console.WriteLine("The following parameters will be used:");
            Console.WriteLine("  Discoverability: Normal");
            Console.WriteLine("  Autonomous Group Owner: Disabled");
            Console.WriteLine("  Legacy mode: Disabled");

            WiFiDirect Adv = new WiFiDirect();
            Adv.Start();
            Console.WriteLine("Press ENTER to terminate");
            Console.ReadLine();
            Adv.Stop();
            Adv = null;

            GC.Collect();
        }
    };
}
